searchData={"items":[{"type":"module","title":"TypedStructNimbleOptions","doc":"Merge typed_struct and nimble_options to form a documented, validated struct type.","ref":"TypedStructNimbleOptions.html"},{"type":"module","title":"Supported options - TypedStructNimbleOptions","doc":"* `:ctor` - Name of the non-bang constructor function.\n  `nil` disables constructor generation. The default value is `:new`.\n\n* `:ctor!` - Name of the bang constructor function.\n  `nil` disables bang constructor generation. The default value is `:new!`.\n\n* `:docs` - Name of the functions that return the NimbleOptions docs for the struct.\n  `nil` disables docs functions generation. The default value is `:field_docs`.\n\n* `:otp_app` (`t:atom/0`) - Name of the OTP application using TypedStructNimbleOptions.\n  This is used to fetch global options for the TypedStructNimbleOptions.\n  Defaults to `Application.get_application(__CALLER__.module)`.\n\n* `:append_moduledoc_header` - Whether to append the generated docs to the @moduledoc.\n  The docs are appended with the header specified in this option.\n  Docs are appended only if the header is not nil. The default value is `\"\\n","ref":"TypedStructNimbleOptions.html#module-supported-options"},{"type":"module","title":"Fields\"`. - TypedStructNimbleOptions","doc":"* `:warn_unknown_types?` (`t:boolean/0`) - Whether to warn when an unknown type is encountered. The default value is `true`.","ref":"TypedStructNimbleOptions.html#module-fields"},{"type":"macro","title":"TypedStructNimbleOptions.after_definition/0","doc":"","ref":"TypedStructNimbleOptions.html#after_definition/0"},{"type":"extras","title":"TypedStructNimbleOptions","doc":"# TypedStructNimbleOptions\n\n[![CI](https://github.com/kzemek/typed_struct_nimble_options/actions/workflows/elixir.yml/badge.svg)](https://github.com/kzemek/typed_struct_nimble_options/actions/workflows/elixir.yml)\n[![Module Version](https://img.shields.io/hexpm/v/typed_struct_nimble_options.svg)](https://hex.pm/packages/typed_struct_nimble_options)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/typed_struct_nimble_options/)\n[![License](https://img.shields.io/hexpm/l/typed_struct_nimble_options.svg)](https://github.com/kzemek/typed_struct_nimble_options/blob/master/LICENSE)\n\n**TypedStructNimbleOptions** is a plugin for [TypedStruct](https://hex.pm/packages/typed_struct) that allows to to easily type, validate & document Elixir structs, all in one place and with little boilerplate.\n\nIt leverages [NimbleOptions](https://hex.pm/packages/nimble_options) for validation & documentation.\nEach `field` of TypedStruct is a key in the generated NimbleOptions schema.\nMany options on `field` are passed directly to NimbleOptions, but in most cases they're automatically derived from the type (and other TypedStruct options).\n\nFor example\n\n```elixir\nfield :attrs, %{optional(atom()) => String.t()}, enforce: true, doc: \"User attributes.\"\n```\n\nwill generate and validate with the following NimbleOptions schema:\n\n```elixir\nattrs: [\n  type: {:map, :atom, :string}, # automatically derived from type\n  required: true, # due to enforce: true\n  doc: \"User attributes.\",\n  type_spec: quote(do: %{optional(atom()) => String.t()})\n]\n```","ref":"readme.html"},{"type":"extras","title":"Example - TypedStructNimbleOptions","doc":"```elixir\ndefmodule Person do\n  @moduledoc \"A struct representing a person.\"\n  @moduledoc since: \"0.1.0\"\n\n  use TypedStruct\n\n  typedstruct do\n    plugin TypedStructNimbleOptions\n\n    field :name, String.t(), enforce: true, type: :string, doc: \"The name.\"\n    field :age, non_neg_integer(), type: :non_neg_integer, doc: \"The age.\"\n    field :happy?, boolean(), default: true\n    field :attrs, %{optional(atom()) => String.t()}\n  end\nend\n\n# `new/1` returns {:ok, value} | {:error, reason}\niex> Person.new(name: \"George\", age: 31)\n{:ok, %Person{name: \"George\", age: 31, happy?: true, attrs: nil}}\n\n# `new!/1` raises on error\niex> Person.new!(name: \"George\", age: 31, attrs: %{phone: 123})\n** (NimbleOptions.ValidationError) invalid map in :attrs option: invalid value for map key :phone: expected string, got: 123\n\n# `field_docs/0-1` returns the fields' documentation\niex> Person.field_docs()\n\"\"\"\n* `:name` (`t:String.t/0`) - Required. The name.\\n\n* `:age` (`t:non_neg_integer/0`) - The age.\\n\n* `:happy?` (`t:boolean/0`) - The default value is `true`.\\n\n* `:attrs` (map of `t:atom/0` keys and `t:String.t/0` values)\\n\n\"\"\"\n```","ref":"readme.html#example"},{"type":"extras","title":"Generated `@moduledoc`s - TypedStructNimbleOptions","doc":"The available options will append themselves to the `@moduledoc`.\nYou can modify this behavior with `append_moduledoc_header` option.\n\nFor example, the ExDoc page for the `Person` module above would start with:\n\n```markdown\n# Person\n\nA struct representing a person.","ref":"readme.html#generated-moduledoc-s"},{"type":"extras","title":"Fields - TypedStructNimbleOptions","doc":"- `:phone` (`String.t/0`)\n\n- `:happy?` (`boolean/0`) - The default value is `true`.\n\n- `:age` (`non_neg_integer/0`) - The age.\n\n- `:name` (`String.t/0`) - Required. The name.\n```","ref":"readme.html#fields"},{"type":"extras","title":"Installation - TypedStructNimbleOptions","doc":"The package can be installed by adding `typed_struct_nimble_options` to your list of dependencies in `mix.exs`:\n\n```elixir\n# mix.exs\n\ndef deps do\n  [\n    {:typed_struct_nimble_options, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Global settings - TypedStructNimbleOptions","doc":"Settings can be specified in application config to apply to all of application's\nTypedStructNimbleOptions structs by default.\n\nFor example, to disable defining the documentation function by default, you can add the following setting:\n\n```elixir\n# config/config.exs\nconfig :my_otp_app, TypedStructNimbleOptions, docs: nil\n```\n\nIndividually, the same settings can be set directly on `plugin TypedStructNimbleOptions` and they will override the default settings if given.\n\nFor example, the below docs setting will override the `config.exs` setting above:\n\n```elixir\ntypedstruct do\n  plugin TypedStructNimbleOptions, docs: :field_docs\nend\n```","ref":"readme.html#global-settings"},{"type":"extras","title":"Supported settings - TypedStructNimbleOptions","doc":"- `:ctor` - Name of the non-bang constructor function. `nil` disables constructor generation. The default value is `:new`.\n- `:ctor!` - Name of the bang constructor function. `nil` disables bang constructor generation. The default value is `:new!`.\n- `:docs` - Name of the functions that return the NimbleOptions docs for the struct. `nil` disables docs functions generation. The default value is `:field_docs`.\n- `:otp_app` (atom/0) - Name of the OTP application using TypedStructNimbleOptions. This is used to fetch global options for the TypedStructNimbleOptions. Defaults to `Application.get_application(__CALLER__.module)`.\n- `:append_moduledoc_header` - Whether to append the generated docs to the `@moduledoc`. The docs are appended with the header specified in this option. Docs are appended only if the header is not nil. The default value is `\"\\n","ref":"readme.html#supported-settings"},{"type":"extras","title":"Fields\"`. - TypedStructNimbleOptions","doc":"- `:warn_unknown_types?` (boolean/0) - Whether to warn when an unknown type is encountered. The default value is `true`.","ref":"readme.html#fields"},{"type":"extras","title":"Field options - TypedStructNimbleOptions","doc":"All of the following options are passed to NimbleOptions as-is, with the exception of `validation_type` which is renamed to `type` as it's being passed down.\n\nSee https://hexdocs.pm/nimble_options/NimbleOptions.html#module-schema-options for more information on the supported options.","ref":"readme.html#field-options"},{"type":"extras","title":"Automatically derived - TypedStructNimbleOptions","doc":"These options are derived from information given to TypedStruct.\nUsers can override the settings by specifying them manually.\n\n- `:required` - set to true if the field is `enforced` and has no `default`.\n- `:type_spec` - set to the field's type\n- `:validation_type` (`type`) - many basic types supported by NimbleOptions are automatically derived from the field's type, for example `atom()` type will set `type: :atom` and `String.t()` will set `type: :string`.\n  Some more complex types like maps or lists are also supported.\n\n  If TypedStructNimbleOptions encounters a type it cannot derive, it will fall back to `:any` and generate a compilation warning.\n  The warnings can be disabled on a struct or global level with `warn_unknown_types?: false`.","ref":"readme.html#automatically-derived"},{"type":"extras","title":"Passed as-is if they're given - TypedStructNimbleOptions","doc":"- `:default`\n- `:keys`\n- `:deprecated`\n- `:doc`\n- `:type_doc`\n- `subsection`","ref":"readme.html#passed-as-is-if-they-re-given"},{"type":"extras","title":"Nested structs - TypedStructNimbleOptions","doc":"Nested structs in particular are not automagically validated.\nYou should either specify `validation_type: {:struct, MyNestedStruct}` manually for the struct field, or run a custom validator with `validation_type: {:custom, M, :f, []}`.\n\nIf the nested struct also uses TypedStructNimbleOptions, you can use a special `validation_type: {:nested_struct, MyNestedStruct, :new}` that will internally run `MyNestedStruct.new/1` with proper error handling:\n\n```elixir\ndefmodule Profile do\n  use TypedStruct\n\n  typedstruct enforce: true do\n    plugin TypedStructNimbleOptions\n    field :name, String.t()\n  end\nend\n\ndefmodule User do\n  use TypedStruct\n\n  typedstruct enforce: true do\n    plugin TypedStructNimbleOptions\n    field :id, pos_integer()\n    field :profile, Profile.t(), validation_type: {:nested_struct, Profile, :new}\n  end\nend\n\niex> User.new!(id: 1, profile: [name: \"UserName\"])\n%User{id: 1, profile: %Profile{name: \"UserName\"}}\n\niex> User.new!(id: 2, profile: %Profile{name: \"UserName\"})\n%User{id: 2, profile: %Profile{name: \"UserName\"}}\n\niex> User.new(id: 3, profile: [name: 404])\n{:error,\n  %NimbleOptions.ValidationError{\n    key: :profile, value: [name: 404], keys_path: [],\n    message: \"invalid value for :profile option: invalid value for :name option: expected string, got: 404\"}}\n```","ref":"readme.html#nested-structs"},{"type":"extras","title":"Limitations - TypedStructNimbleOptions","doc":"","ref":"readme.html#limitations"},{"type":"extras","title":"Compilation-time schema - TypedStructNimbleOptions","doc":"The NimbleOptions schema is prepared at the compilation time, so it cannot contain runtime elements such as `fn` definitions.","ref":"readme.html#compilation-time-schema"},{"type":"extras","title":"Automatically derived types - TypedStructNimbleOptions","doc":"Non-parametrized types can be given with or without parentheses, e.g. `map()` is equivalent to `map`.\n\n| Elixir type                                    | NimbleOption type                    |\n| ---------------------------------------------- | ------------------------------------ |\n| `map()`, `%{}`                                 | `{:map, :any, :any}`                 |\n| `%{optional(key) => value}`, `%{key => value}` | `{:map, derive(key), derive(value)}` |\n| `list()`                                       | `{:list, :any}`                      |\n| `list(subtype)`, `[subtype]`                   | `{:list, derive(subtype)}`           |\n| `{typea, ...}`                                 | `{:tuple, [derive(typea), ...]}`     |\n| `atom()`                                       | `:atom`                              |\n| `String.t()`                                   | `:string`                            |\n| `boolean()`                                    | `:boolean`                           |\n| `integer()`                                    | `:integer`                           |\n| `non_neg_integer()`                            | `:non_neg_integer`                   |\n| `pos_integer()`                                | `:pos_integer`                       |\n| `float()`                                      | `:float`                             |\n| `timeout()`                                    | `:timeout`                           |\n| `pid()`                                        | `:pid`                               |\n| `reference()`                                  | `:reference`                         |\n| `nil`                                          | `nil`                                |\n| `mfa()`                                        | `:mfa`                               |\n| `any()`, `term()`                              | `:any`                               |","ref":"readme.html#automatically-derived-types"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}